@@ -4,11 +4,13 @@ import numpy as np
 import pandas as pd
 import six
 
+from pyproj import CRS
 import shapely
 import shapely.affinity
 import shapely.geometry
 from shapely.geometry.base import CAP_STYLE, JOIN_STYLE
 import shapely.wkb
+from shapely._buildcfg import geos_version
 
 import geopandas
 from geopandas.array import (
@@ -19,7 +21,10 @@ from geopandas.array import (
     points_from_xy,
     to_wkb,
     to_wkt,
+    _check_crs,
+    _crs_mismatch_warn,
 )
+import geopandas._compat as compat
 
 import pytest
 
@@ -48,8 +53,8 @@ def equal_geometries(result, expected):
 
 
 def test_points():
-    x = np.arange(10).astype(np.float)
-    y = np.arange(10).astype(np.float) ** 2
+    x = np.arange(10).astype(np.float64)
+    y = np.arange(10).astype(np.float64) ** 2
 
     points = points_from_xy(x, y)
     assert isinstance(points, GeometryArray)
@@ -69,15 +74,19 @@ def test_points_from_xy():
     gsz = [shapely.geometry.Point(x, x, x) for x in range(10)]
     geometry1 = geopandas.points_from_xy(df["x"], df["y"])
     geometry2 = geopandas.points_from_xy(df["x"], df["y"], df["z"])
-    assert geometry1 == gs
-    assert geometry2 == gsz
+    assert isinstance(geometry1, GeometryArray)
+    assert isinstance(geometry2, GeometryArray)
+    assert list(geometry1) == gs
+    assert list(geometry2) == gsz
 
     # using Series or numpy arrays or lists
     for s in [pd.Series(range(10)), np.arange(10), list(range(10))]:
         geometry1 = geopandas.points_from_xy(s, s)
         geometry2 = geopandas.points_from_xy(s, s, s)
-        assert geometry1 == gs
-        assert geometry2 == gsz
+        assert isinstance(geometry1, GeometryArray)
+        assert isinstance(geometry2, GeometryArray)
+        assert list(geometry1) == gs
+        assert list(geometry2) == gsz
 
     # using different lengths should throw error
     arr_10 = np.arange(10)
@@ -109,9 +118,11 @@ def test_from_shapely_geo_interface():
             return {"type": "Point", "coordinates": (self.x, self.y)}
 
     result = from_shapely([Point(1.0, 2.0), Point(3.0, 4.0)])
+
     expected = from_shapely(
         [shapely.geometry.Point(1.0, 2.0), shapely.geometry.Point(3.0, 4.0)]
     )
+
     assert all(v.equals(t) for v, t in zip(result, expected))
 
 
@@ -128,10 +139,15 @@ def test_from_wkb():
     assert all(v.equals(t) for v, t in zip(res, points_no_missing))
 
     # missing values
-    L_wkb.extend([b"", None])
+    # TODO(pygeos) does not support empty strings
+    if compat.USE_PYGEOS:
+        L_wkb.extend([None])
+    else:
+        L_wkb.extend([b"", None])
     res = from_wkb(L_wkb)
     assert res[-1] is None
-    assert res[-2] is None
+    if not compat.USE_PYGEOS:
+        assert res[-2] is None
 
     # single MultiPolygon
     multi_poly = shapely.geometry.MultiPolygon(
@@ -148,6 +164,11 @@ def test_to_wkb():
     assert isinstance(res, np.ndarray)
     np.testing.assert_array_equal(res, exp)
 
+    res = to_wkb(P, hex=True)
+    exp = np.array([p.wkb_hex for p in points_no_missing], dtype=object)
+    assert isinstance(res, np.ndarray)
+    np.testing.assert_array_equal(res, exp)
+
     # missing values
     a = from_shapely([None, points_no_missing[0]])
     res = to_wkb(a)
@@ -181,10 +202,15 @@ def test_from_wkt(string_type):
     assert all(v.almost_equals(t) for v, t in zip(res, points_no_missing))
 
     # missing values
-    L_wkt.extend([f(""), None])
+    # TODO(pygeos) does not support empty strings
+    if compat.USE_PYGEOS:
+        L_wkt.extend([None])
+    else:
+        L_wkt.extend([f(""), None])
     res = from_wkt(L_wkt)
     assert res[-1] is None
-    assert res[-2] is None
+    if not compat.USE_PYGEOS:
+        assert res[-2] is None
 
     # single MultiPolygon
     multi_poly = shapely.geometry.MultiPolygon(
@@ -196,7 +222,7 @@ def test_from_wkt(string_type):
 
 def test_to_wkt():
     P = from_shapely(points_no_missing)
-    res = to_wkt(P)
+    res = to_wkt(P, rounding_precision=-1)
     exp = np.array([p.wkt for p in points_no_missing], dtype=object)
     assert isinstance(res, np.ndarray)
     np.testing.assert_array_equal(res, exp)
@@ -214,13 +240,13 @@ def test_to_wkt():
         ("covers", ()),
         ("crosses", ()),
         ("disjoint", ()),
-        ("equals", ()),
+        ("geom_equals", ()),
         ("intersects", ()),
         ("overlaps", ()),
         ("touches", ()),
         ("within", ()),
-        ("equals_exact", (0.1,)),
-        ("almost_equals", (3,)),
+        ("geom_equals_exact", (0.1,)),
+        ("geom_almost_equals", (3,)),
     ],
 )
 def test_predicates_vector_scalar(attr, args):
@@ -235,7 +261,9 @@ def test_predicates_vector_scalar(attr, args):
         assert result.dtype == bool
 
         expected = [
-            getattr(tri, attr)(other, *args) if tri is not None else na_value
+            getattr(tri, attr if "geom" not in attr else attr[5:])(other, *args)
+            if tri is not None
+            else na_value
             for tri in triangles
         ]
 
@@ -251,13 +279,13 @@ def test_predicates_vector_scalar(attr, args):
         ("covers", ()),
         ("crosses", ()),
         ("disjoint", ()),
-        ("equals", ()),
+        ("geom_equals", ()),
         ("intersects", ()),
         ("overlaps", ()),
         ("touches", ()),
         ("within", ()),
-        ("equals_exact", (0.1,)),
-        ("almost_equals", (3,)),
+        ("geom_equals_exact", (0.1,)),
+        ("geom_almost_equals", (3,)),
     ],
 )
 def test_predicates_vector_vector(attr, args):
@@ -293,11 +321,26 @@ def test_predicates_vector_vector(attr, args):
         elif a.is_empty or b.is_empty:
             expected.append(empty_value)
         else:
-            expected.append(getattr(a, attr)(b, *args))
+            expected.append(
+                getattr(a, attr if "geom" not in attr else attr[5:])(b, *args)
+            )
 
     assert result.tolist() == expected
 
 
+@pytest.mark.parametrize(
+    "attr,args", [("equals_exact", (0.1,)), ("almost_equals", (3,))]
+)
+def test_equals_deprecation(attr, args):
+    point = points[0]
+    tri = triangles[0]
+
+    for other in [point, tri, shapely.geometry.Polygon()]:
+        with pytest.warns(FutureWarning):
+            result = getattr(T, attr)(other, *args)
+        assert result.tolist() == getattr(T, "geom_" + attr)(other, *args).tolist()
+
+
 @pytest.mark.parametrize(
     "attr",
     [
@@ -313,9 +356,11 @@ def test_unary_geo(attr):
     na_value = None
 
     if attr == "boundary":
-        # boundary raises for empty geometry
-        with pytest.raises(Exception):
-            T.boundary
+        # pygeos returns None for empty geometries
+        if not compat.USE_PYGEOS:
+            # boundary raises for empty geometry
+            with pytest.raises(Exception):
+                T.boundary
 
         values = triangle_no_missing + [None]
         A = from_shapely(values)
@@ -324,7 +369,17 @@ def test_unary_geo(attr):
         A = T
 
     result = getattr(A, attr)
-    expected = [getattr(t, attr) if t is not None else na_value for t in values]
+    if attr == "exterior" and compat.USE_PYGEOS:
+        # TODO(pygeos)
+        # empty Polygon() has an exterior with shapely > 1.7, which gives
+        # empty LinearRing instead of None,
+        # but conversion to pygeos still results in empty GeometryCollection
+        expected = [
+            getattr(t, attr) if t is not None and not t.is_empty else na_value
+            for t in values
+        ]
+    else:
+        expected = [getattr(t, attr) if t is not None else na_value for t in values]
 
     assert equal_geometries(result, expected)
 
@@ -394,8 +449,8 @@ def test_binary_geo_scalar(attr):
 )
 def test_unary_predicates(attr):
     na_value = False
-    if attr == "is_simple":
-        # poly.is_simple raises an error for empty polygon
+    if attr == "is_simple" and geos_version < (3, 8) and not compat.USE_PYGEOS:
+        # poly.is_simple raises an error for empty polygon for GEOS < 3.8
         with pytest.raises(Exception):
             T.is_simple
         vals = triangle_no_missing
@@ -406,7 +461,16 @@ def test_unary_predicates(attr):
 
     result = getattr(V, attr)
 
-    if attr == "is_ring":
+    if attr == "is_simple" and (geos_version < (3, 8) or compat.USE_PYGEOS):
+        # poly.is_simple raises an error for empty polygon for GEOS < 3.8
+        # with shapely, pygeos always returns False for all GEOS versions
+        # But even for Shapely with GEOS >= 3.8, empty GeometryCollection
+        # returns True instead of False
+        expected = [
+            getattr(t, attr) if t is not None and not t.is_empty else na_value
+            for t in vals
+        ]
+    elif attr == "is_ring":
         expected = [
             getattr(t.exterior, attr)
             if t is not None and t.exterior is not None
@@ -418,12 +482,28 @@ def test_unary_predicates(attr):
     assert result.tolist() == expected
 
 
+def test_is_ring():
+    g = [
+        shapely.geometry.LinearRing([(0, 0), (1, 1), (1, -1)]),
+        shapely.geometry.LineString([(0, 0), (1, 1), (1, -1)]),
+        shapely.geometry.LineString([(0, 0), (1, 1), (1, -1), (0, 0)]),
+        shapely.geometry.Polygon([(0, 0), (1, 1), (1, -1)]),
+        shapely.geometry.Polygon(),
+        None,
+    ]
+    expected = [True, False, True, True, False, False]
+
+    result = from_shapely(g).is_ring
+
+    assert result.tolist() == expected
+
+
 @pytest.mark.parametrize("attr", ["area", "length"])
 def test_unary_float(attr):
     na_value = np.nan
     result = getattr(T, attr)
     assert isinstance(result, np.ndarray)
-    assert result.dtype == np.float
+    assert result.dtype == np.dtype("float64")
     expected = [getattr(t, attr) if t is not None else na_value for t in triangles]
     np.testing.assert_allclose(result, expected)
 
@@ -526,6 +606,11 @@ def test_binary_project(normalized):
 @pytest.mark.parametrize("join_style", [JOIN_STYLE.round, JOIN_STYLE.bevel])
 @pytest.mark.parametrize("resolution", [16, 25])
 def test_buffer(resolution, cap_style, join_style):
+    if compat.USE_PYGEOS:
+        # TODO(pygeos) need to further investigate why this test fails
+        if cap_style == 1 and join_style == 3:
+            pytest.skip("failing TODO")
+
     na_value = None
     expected = [
         p.buffer(0.1, resolution=resolution, cap_style=cap_style, join_style=join_style)
@@ -536,7 +621,12 @@ def test_buffer(resolution, cap_style, join_style):
     result = P.buffer(
         0.1, resolution=resolution, cap_style=cap_style, join_style=join_style
     )
+    assert equal_geometries(expected, result)
 
+    dist = np.array([0.1] * len(P))
+    result = P.buffer(
+        dist, resolution=resolution, cap_style=cap_style, join_style=join_style
+    )
     assert equal_geometries(expected, result)
 
 
@@ -616,6 +706,36 @@ def test_bounds():
         assert result.dtype == "float64"
         np.testing.assert_allclose(result, np.array([[np.nan] * 4]))
 
+    # empty array (https://github.com/geopandas/geopandas/issues/1195)
+    E = from_shapely([])
+    result = E.bounds
+    assert result.shape == (0, 4)
+    assert result.dtype == "float64"
+
+
+def test_total_bounds():
+    result = T.total_bounds
+    bounds = np.array(
+        [t.bounds if not (t is None or t.is_empty) else [np.nan] * 4 for t in triangles]
+    )
+    expected = np.array(
+        [
+            np.nanmin(bounds[:, 0]),  # minx
+            np.nanmin(bounds[:, 1]),  # miny
+            np.nanmax(bounds[:, 2]),  # maxx
+            np.nanmax(bounds[:, 3]),  # maxy
+        ]
+    )
+    np.testing.assert_allclose(result, expected)
+
+    # additional check for empty array or one empty / missing
+    for geoms in [[], [None], [shapely.geometry.Polygon()]]:
+        E = from_shapely(geoms)
+        result = E.total_bounds
+        assert result.ndim == 1
+        assert result.dtype == "float64"
+        np.testing.assert_allclose(result, np.array([np.nan] * 4))
+
 
 def test_getitem():
     points = [shapely.geometry.Point(i, i) for i in range(10)]
@@ -652,6 +772,15 @@ def test_equality_ops():
     res = a1 != a2
     assert res.tolist() == [False, True, False]
 
+    # check the correct expansion of list-like geometry
+    multi_poly = shapely.geometry.MultiPolygon(
+        [shapely.geometry.box(0, 0, 1, 1), shapely.geometry.box(3, 3, 4, 4)]
+    )
+    a3 = from_shapely([points[1], points[2], points[3], multi_poly])
+
+    res = a3 == multi_poly
+    assert res.tolist() == [False, False, False, True]
+
 
 def test_dir():
     assert "contains" in dir(P)
@@ -671,7 +800,7 @@ def test_pickle():
     # assert (T.data != T2.data).all()
     assert T2[-1] is None
     assert T2[-2].is_empty
-    assert T[:-2].equals(T2[:-2]).all()
+    assert T[:-2].geom_equals(T2[:-2]).all()
 
 
 def test_raise_on_bad_sizes():
@@ -714,3 +843,96 @@ def test_astype_multipolygon():
     result = arr.astype(np.dtype("U10"))
     assert result.dtype == np.dtype("U10")
     assert result[0] == multi_poly.wkt[:10]
+
+
+def test_check_crs():
+    t1 = T.copy()
+    t1.crs = 4326
+    assert _check_crs(t1, T) is False
+    assert _check_crs(t1, t1) is True
+    assert _check_crs(t1, T, allow_none=True) is True
+
+
+def test_crs_mismatch_warn():
+    t1 = T.copy()
+    t2 = T.copy()
+    t1.crs = 4326
+    t2.crs = 3857
+
+    # two different CRS
+    with pytest.warns(UserWarning, match="CRS mismatch between the CRS"):
+        _crs_mismatch_warn(t1, t2)
+
+    # left None
+    with pytest.warns(UserWarning, match="CRS mismatch between the CRS"):
+        _crs_mismatch_warn(T, t2)
+
+    # right None
+    with pytest.warns(UserWarning, match="CRS mismatch between the CRS"):
+        _crs_mismatch_warn(t1, T)
+
+
+@pytest.mark.parametrize("NA", [None, np.nan])
+def test_isna(NA):
+    t1 = T.copy()
+    t1[0] = NA
+    assert t1[0] is None
+
+
+@pytest.mark.skipif(not compat.PANDAS_GE_10, reason="pd.NA introduced in pandas 1.0")
+def test_isna_pdNA():
+    t1 = T.copy()
+    t1[0] = pd.NA
+    assert t1[0] is None
+
+
+def test_shift_has_crs():
+    t = T.copy()
+    t.crs = 4326
+    assert t.shift(1).crs == t.crs
+    assert t.shift(0).crs == t.crs
+    assert t.shift(-1).crs == t.crs
+
+
+@pytest.mark.skipif(
+    not compat.PANDAS_GE_115, reason="crs only preserved in unique after pandas 1.1.5"
+)
+def test_unique_has_crs():
+    t = T.copy()
+    t.crs = 4326
+    assert t.unique().crs == t.crs
+
+
+class TestEstimateUtmCrs:
+    def setup_method(self):
+        self.esb = shapely.geometry.Point(-73.9847, 40.7484)
+        self.sol = shapely.geometry.Point(-74.0446, 40.6893)
+        self.landmarks = from_shapely([self.esb, self.sol], crs="epsg:4326")
+
+    def test_estimate_utm_crs__geographic(self):
+        if compat.PYPROJ_LT_3:
+            with pytest.raises(RuntimeError, match=r"pyproj 3\+ required"):
+                self.landmarks.estimate_utm_crs()
+        else:
+            assert self.landmarks.estimate_utm_crs() == CRS("EPSG:32618")
+            assert self.landmarks.estimate_utm_crs("NAD83") == CRS("EPSG:26918")
+
+    @pytest.mark.skipif(compat.PYPROJ_LT_3, reason="requires pyproj 3 or higher")
+    def test_estimate_utm_crs__projected(self):
+        assert self.landmarks.to_crs("EPSG:3857").estimate_utm_crs() == CRS(
+            "EPSG:32618"
+        )
+
+    @pytest.mark.skipif(compat.PYPROJ_LT_3, reason="requires pyproj 3 or higher")
+    def test_estimate_utm_crs__out_of_bounds(self):
+        with pytest.raises(RuntimeError, match="Unable to determine UTM CRS"):
+            from_shapely(
+                [shapely.geometry.Polygon([(0, 90), (1, 90), (2, 90)])], crs="EPSG:4326"
+            ).estimate_utm_crs()
+
+    @pytest.mark.skipif(compat.PYPROJ_LT_3, reason="requires pyproj 3 or higher")
+    def test_estimate_utm_crs__missing_crs(self):
+        with pytest.raises(RuntimeError, match="crs must be set"):
+            from_shapely(
+                [shapely.geometry.Polygon([(0, 90), (1, 90), (2, 90)])]
+            ).estimate_utm_crs()
