@@ -1,11 +1,8 @@
 from collections import defaultdict
 import time
 
-import numpy as np
 import pandas as pd
-from six import iteritems, string_types
 
-from fiona.crs import from_epsg
 from shapely.geometry import Point
 
 import geopandas
@@ -55,14 +52,13 @@ def geocode(strings, provider=None, **kwargs):
 
     Examples
     --------
-    >>> df = geocode(['boston, ma', '1600 pennsylvania ave. washington, dc'])
-    >>> df
-                                                 address  \\
-    0                                    Boston, MA, USA
-    1  1600 Pennsylvania Avenue Northwest, President'...
-                             geometry
-    0  POINT (-71.0597732 42.3584308)
-    1  POINT (-77.0365305 38.8977332)
+    >>> df = geopandas.tools.geocode(  # doctest: +SKIP
+    ...         ["boston, ma", "1600 pennsylvania ave. washington, dc"]
+    ...     )
+    >>> df  # doctest: +SKIP
+                        geometry                                            address
+    0  POINT (-71.05863 42.35899)                          Boston, MA, United States
+    1  POINT (-77.03651 38.89766)  1600 Pennsylvania Ave NW, Washington, DC 20006...
     """
 
     if provider is None:
@@ -110,15 +106,14 @@ def reverse_geocode(points, provider=None, **kwargs):
 
     Examples
     --------
-    >>> df = reverse_geocode([Point(-71.0594869, 42.3584697),
-                              Point(-77.0365305, 38.8977332)])
-    >>> df
-                                             address  \\
-    0             29 Court Square, Boston, MA 02108, USA
-    1  1600 Pennsylvania Avenue Northwest, President'...
-                             geometry
-    0  POINT (-71.0594869 42.3584697)
-    1  POINT (-77.0365305 38.8977332)
+    >>> from shapely.geometry import Point
+    >>> df = geopandas.tools.reverse_geocode(  # doctest: +SKIP
+    ...     [Point(-71.0594869, 42.3584697), Point(-77.0365305, 38.8977332)]
+    ... )
+    >>> df  # doctest: +SKIP
+                         geometry                                            address
+    0  POINT (-71.05941 42.35837)       29 Court Sq, Boston, MA 02108, United States
+    1  POINT (-77.03641 38.89766)  1600 Pennsylvania Ave NW, Washington, DC 20006...
     """
 
     if provider is None:
@@ -139,12 +134,12 @@ def _query(data, forward, provider, throttle_time, **kwargs):
     if not isinstance(data, pd.Series):
         data = pd.Series(data)
 
-    if isinstance(provider, string_types):
+    if isinstance(provider, str):
         provider = get_geocoder_for_service(provider)
 
     coder = provider(**kwargs)
     results = {}
-    for i, s in iteritems(data):
+    for i, s in data.items():
         try:
             if forward:
                 results[i] = coder.geocode(s)
@@ -170,23 +165,25 @@ def _prepare_geocode_result(results):
     d = defaultdict(list)
     index = []
 
-    for i, s in iteritems(results):
-        address, loc = s
+    for i, s in results.items():
 
-        # loc is lat, lon and we want lon, lat
-        if loc is None:
+        if s is None:
             p = Point()
-        else:
-            p = Point(loc[1], loc[0])
+            address = None
 
-        if address is None:
-            address = np.nan
+        else:
+            address, loc = s
+
+            # loc is lat, lon and we want lon, lat
+            if loc is None:
+                p = Point()
+            else:
+                p = Point(loc[1], loc[0])
 
         d["geometry"].append(p)
         d["address"].append(address)
         index.append(i)
 
-    df = geopandas.GeoDataFrame(d, index=index)
-    df.crs = from_epsg(4326)
+    df = geopandas.GeoDataFrame(d, index=index, crs="EPSG:4326")
 
     return df
